/*
 * Copyright 2017-2020 吴学文 and java110 team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.java110.store.cmd.classes;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.java110.core.annotation.Java110Cmd;
import com.java110.core.annotation.Java110Transactional;
import com.java110.core.context.ICmdDataFlowContext;
import com.java110.core.event.cmd.Cmd;
import com.java110.core.event.cmd.CmdEvent;
import com.java110.core.factory.GenerateCodeFactory;
import com.java110.intf.store.IClassesTimeV1InnerServiceSMO;
import com.java110.intf.store.IClassesV1InnerServiceSMO;
import com.java110.po.classes.ClassesPo;
import com.java110.po.classesTime.ClassesTimePo;
import com.java110.utils.exception.CmdException;
import com.java110.utils.util.Assert;
import com.java110.utils.util.BeanConvertUtil;
import com.java110.vo.ResultVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * 类表述：更新
 * 服务编码：classes.updateClasses
 * 请求路劲：/app/classes.UpdateClasses
 * add by 吴学文 at 2022-11-01 00:27:10 mail: 928255095@qq.com
 * open source address: https://gitee.com/wuxw7/MicroCommunity
 * 官网：http://www.homecommunity.cn
 * 温馨提示：如果您对此文件进行修改 请不要删除原有作者及注释信息，请补充您的 修改的原因以及联系邮箱如下
 * // modify by 张三 at 2021-09-12 第10行在某种场景下存在某种bug 需要修复，注释10至20行 加入 20行至30行
 */
@Java110Cmd(serviceCode = "classes.updateClasses")
public class UpdateClassesCmd extends Cmd {

  private static Logger logger = LoggerFactory.getLogger(UpdateClassesCmd.class);


    @Autowired
    private IClassesV1InnerServiceSMO classesV1InnerServiceSMOImpl;


    @Autowired
    private IClassesTimeV1InnerServiceSMO classesTimeV1InnerServiceSMOImpl;

    @Override
    public void validate(CmdEvent event, ICmdDataFlowContext cmdDataFlowContext, JSONObject reqJson) {
        Assert.hasKeyAndValue(reqJson, "classesId", "classesId不能为空");

        if(!reqJson.containsKey("times")){
            throw new CmdException("未包含时间");
        }

        JSONArray times = reqJson.getJSONArray("times");

        if(times.size() < 1){
            throw new CmdException("未包含时间");
        }
    }

    @Override
    @Java110Transactional
    public void doCmd(CmdEvent event, ICmdDataFlowContext cmdDataFlowContext, JSONObject reqJson) throws CmdException {

       ClassesPo classesPo = BeanConvertUtil.covertBean(reqJson, ClassesPo.class);
        int flag = classesV1InnerServiceSMOImpl.updateClasses(classesPo);

        if (flag < 1) {
            throw new CmdException("更新数据失败");
        }
        ClassesTimePo classesTimePo = null;
        classesTimePo = new ClassesTimePo();
        classesTimePo.setClassesId(classesPo.getClassesId());
        flag = classesTimeV1InnerServiceSMOImpl.deleteClassesTime(classesTimePo);
        if (flag < 1) {
            throw new CmdException("更新数据失败");
        }
        JSONArray times = reqJson.getJSONArray("times");
        JSONObject time = null;

        for(int timeIndex = 0; timeIndex < times.size() ; timeIndex ++){
            time = times.getJSONObject(timeIndex);
            classesTimePo = new ClassesTimePo();
            classesTimePo.setClassesId(classesPo.getClassesId());
            classesTimePo.setStartTime(time.getString("startTime"));
            classesTimePo.setEndTime(time.getString("endTime"));
            classesTimePo.setTimeId(GenerateCodeFactory.getGeneratorId("11"));
            flag = classesTimeV1InnerServiceSMOImpl.saveClassesTime(classesTimePo);
            if (flag < 1) {
                throw new CmdException("保存数据失败");
            }
        }

        cmdDataFlowContext.setResponseEntity(ResultVo.success());
    }
}
